open Topology
open Message

type t = {
  ptr: int array;
  sz: int array;
  topo: topo;
  no : int array;
  mutable classes: int;

  sz_set : int;
}

let rec repr t i =
  let j = t.ptr.(i) in
  if (i = j) then (assert(t.sz.(i) > 0); i)
  else (
    assert(t.sz.(i) < 0);
    let j = repr t j in
    t.ptr.(i) <- j;
    j
  )

let repr t n = repr t n.nod_num

let merge t n1 n2 =
  let i1 = repr t n1 and i2 = repr t n2 in
  if i1 <> i2 then (
    assert(t.sz.(i1) > 0); assert(t.sz.(i2) > 0);
    t.classes <- t.classes - 1;
    t.ptr.(i1) <- i2;
    t.sz.(i2) <- t.sz.(i1) + t.sz.(i2);
    t.sz.(i1) <- (-1);
  )

let decompose topo set =
  let s = Array.length topo.t_nods in
  let t = { ptr = Array.init s (fun i -> i);
	    sz = Array.create s 1; 
	    sz_set = NS.cardinal set;
	    classes = NS.cardinal set;
	    no = Array.create s (-1);
	    topo = topo } in
  NS.iter
    (fun n -> NS.iter (merge t n) (NS.inter set (dist_f_1 topo n))) 
    set;
  let nb = ref 0 in
  for i = 0 to Array.length t.ptr - 1 do
    if (t.ptr.(i) = i) && (NS.mem topo.t_nods.(i) set) then 
      (t.no.(i) <- !nb; incr nb;)
  done;
  Printf.eprintf "%i connected components\n" t.classes;
  assert(!nb = t.classes);
  t

let size t n = t.sz.(repr t n)


let split t tot n =
  tot / (size t n)


let classes t = t.classes

let repr t n = t.no.(repr t n)

