(* Fonctions  utiliser pour coder des policiers *)
open Message
open Topology

type t

type env = {
  env_wsk    : world_skeleton;
  env_topo   : topo;
  mutable env_info   : Info.info;
  env_drawer : Draw.t;
}

(* Liste des deplacements (un pour chaque policier control) *)
type mov = (int * (ptype * nod)) list

(* L'automate *)

val cop_inform: t -> env -> inf list -> (string * inf list) list
val cop_plan: t -> plan list -> (string * plan list) list
val cop_vote: t -> vote list -> vote_res

val cop_r_inform: t -> env -> inf list -> (string * inf list) list
  (* N'intgre pas les infos reues dans l'environnement *)


(* Enregistrement *)

type 'a cops_fun = {
  (* This function is called before the interaction loop. *)
  cops_init : t -> env -> 'a;

  (* This function must implement the game protocol. It is called
     when the cop is clean. The following functions are to be called successively:
     - cop_inform
     - cop_plan
     - cop_vote
     The [int list] in the result is the list of players to accuse.
  *)
  cops_clean_job  : t -> env -> world -> 'a -> (offer * mov * int list * 'a);

  (* This function is called when the cop is dirty. Cop round.
     The following functions are to be called successively:
     - cop_inform
     - cop_plan
     - cop_vote
  *)
  cops_dirty_job  : t -> env -> world -> 'a -> (mov * 'a);

  (* This function is called when the cop is dirty. Robber/scoundrel negociation 
     round. The following functions are to be called successively:
     - cop_inform
     - cop_plan
     - cop_vote
  *)
  cops_dirty_r_job : t -> env -> world -> 'a -> 'a;
  


  (* Initial player type for the cop: Cop_foot or Cop_car. *)
  cops_ptype : ptype;
}

val register_cop: string -> 'a cops_fun -> unit
val run_cop: in_channel -> string -> unit

(*
val dump_mov : (ptype * nod) -> unit
*)
