(* $Id: message.mli,v 1.20.4.3 2005/07/09 19:45:05 frisch Exp $ *)

type ptype =
  | Cop_foot
  | Cop_car
  | Robber
;;

type node_tag =
  | Hq
  | Bank
  | Robber_start
  | Ordinary
;;

type edge_type =
  | Car
  | Foot
;;

(* ---------------------- *)

type nod = {
    nod_num : int;
    nod_loc : string;
    nod_tag : node_tag;
    nod_x : int;
    nod_y : int;
  }

val nnodes : int ref

type edg = {
  edg_src : string;
  edg_dst : string;
  edg_type : edge_type;
}

type bv = {
  bv_loc : string;
  bv_value : int;
}

type ev = {
  ev_loc : string;
  ev_world : int;
}

type pl = {
  pl_bot : string;
  pl_loc : string;
  pl_ptype : ptype;
}

(* ---------------------- *)

type register = {
  reg_bot : string;
  reg_ptype : ptype;
}

type world_skeleton = {
  wsk_name : string;
  wsk_robber : string;
  wsk_cops : string list;
  wsk_nods : nod list;
  wsk_edgs : edg list;
}

type sc = {
  sc_brain: string;
  sc_cop: string;
}

type fac = {
  fac_accusing: string;
  fac_accused: string;
  fac_wor: int;
}

type world = {
  wor_wor : int;
  wor_rbd: int;
  wor_dc: string list; (* known dirty brains *)
  wor_sc: sc list; (* cops taken over *)
  wor_fac: fac list; (* false accusations *)
  wor_bvs: bv list;
  wor_evs: ev list;
  wor_smell: int;
  wor_pls: pl list;
}


type offer = Turncoat | Straight

type move = {
  mov_loc : string;
  mov_ptype : ptype;
  mov_bot : string;
}

type moves = move list

type clean_cop_move = {
  mov_offer: offer;
  mov_moves: moves;
  mov_accuse: string list;
}

type dirty_cop_move = move list

type bribing = Bribe | Nobribe

type offered_cops =
  | NotTelling
  | Offered of string list

type push = {
  push_bot: string;
  push_loc: string;
}

type bribe_result =
  | Push of push
  | Choose of string list
  | Nothing

type robber_move = {
  mov_mov: move;
  mov_bribe: bribe_result;
}

type world_or_game_over =
  | World of world
  | Game_over
;;

type inf = {
  inf_bot : string;
  inf_loc : string;
  inf_ptype : ptype;
  inf_world : int;
  inf_certainty : int;
}

type plan = {
  plan_bot : string;
  plan_loc : string;
  plan_ptype : ptype;
  plan_world : int;
}

type vote = string;;

type vote_res =
  | Winner of string
  | Nowinner
;;



val register : register -> unit
(*val move : move -> unit*)

val clean_cop_move: clean_cop_move -> unit
val dirty_cop_move: dirty_cop_move -> unit
val robber_move: robber_move -> unit
val bribing: bribing -> unit

val vote : vote list -> unit
val plan : plan list -> unit
val inform : inf list -> unit

val tag_to_string : node_tag -> string;;
(* Default value for plan. *)
val default_plan : plan

val is_headquarter : nod -> bool

val is_cop : pl -> bool

val ptype_to_string : ptype -> string
val neg_ptype : ptype -> ptype

val dump_plan : plan -> unit
