(* $Id: misc.ml,v 1.4 2005/06/26 21:11:42 regisgia Exp $ *)

module IntComparable =
struct
  type t = int
  let  compare = (Pervasives.compare : t -> t -> int)
end

module IntSet = Set.Make(IntComparable)
module IntMap = Map.Make(IntComparable)

module F_INT =
struct
  type f_int =
    | F_Inf
    | F_Int of int

  let (++) = fun left right ->
    match left, right with
      | F_Inf  , F_Inf
      | F_Int _, F_Inf
      | F_Inf  , F_Int _ -> F_Inf
      | F_Int x, F_Int y -> F_Int (x+y)

  let (<<=) = fun left right ->
    match left, right with
      | F_Inf  , F_Inf   -> true
      | F_Inf  , F_Int _ -> false
      | F_Int _, F_Inf   -> true
      | F_Int x, F_Int y -> x <= y

  let f_min = fun left right ->
    match left, right with
      | F_Inf  , F_Inf   -> F_Inf
      | F_Int x, F_Inf   -> F_Int x
      | F_Inf  , F_Int y -> F_Int y
      | F_Int x, F_Int y -> F_Int (min x y)

  let int_of_fint = function
    | F_Int x -> x
    | F_Inf   -> max_int
end

let swap = fun v1 v2 ->
  let tmp = !v1 in
    v1 := !v2; v2 := tmp

let apply_opt = fun f -> function
  | None -> None
  | Some v -> Some (f v)
