type token =
  | EOF
  | EOL
  | IDENT of (string)
  | BVCL
  | BV
  | BVOP
  | COP
  | EDGCL
  | EDG
  | EDGOP
  | EVCL
  | EV
  | EVOP
  | FROMCL
  | FROM
  | FROMOP
  | INFCL
  | INF
  | INFOP
  | MOVOP
  | MOVCL
  | MOV
  | NAME
  | NODCL
  | NOD
  | NODOP
  | NOWINNER
  | PLCL
  | PL
  | PLOP
  | PLANCL
  | PLAN
  | PLANOP
  | RBD
  | REG
  | ROBBER
  | SMELL
  | VOTECL
  | VOTE
  | VOTEOP
  | WINNER
  | WORCL
  | WOR
  | WOROP
  | WSKCL
  | WSKOP
  | DCCL
  | DC
  | DCOP
  | SCCL
  | SC
  | SCOP
  | FACCL
  | FAC
  | FACOP
  | TURNCOAT
  | STRAIGHTARROW
  | CMOVOP
  | CMOVCL
  | ACCOP
  | ACCCL
  | ACC
  | OFC
  | OFCOP
  | OFCCL
  | NOTTELLING
  | ACCUSED

open Parsing;;
# 4 "parser.mly"

open Message;;

let mk_ptype = function
  | "cop-foot" -> Cop_foot
  | "cop-car" -> Cop_car
  | "robber" -> Robber
  | _ -> raise Parse_error
;;

let mk_node_tag = function
  | "hq" -> Hq
  | "bank" -> Bank
  | "robber-start" -> Robber_start
  | "ordinary" -> Ordinary
  | _ -> raise Parse_error
;;

let mk_edge_type = function
  | "car" -> Car
  | "foot" -> Foot
  | _ -> raise Parse_error
;;

let mk_game_over = function
  | "game-over" -> Game_over
  | _ -> raise Parse_error
;;

let between lower upper s =
  try
    let n = int_of_string s in
    if n < lower || n > upper then raise Parse_error;
    n
  with Failure _ -> raise Parse_error
;;

let mk_coordinate s = between 0 1023 s;;
let mk_world s = between 0 200 s;;
let mk_loot s = between 0 6000 s;;
let mk_bank_value s = between 0 1000 s;;
let mk_distance s = between 0 1_000_000_000 s;;
let mk_certainty s = between (-100) 100 s;;

# 117 "parser.ml"
let yytransl_const = [|
    0 (* EOF *);
  257 (* EOL *);
  259 (* BVCL *);
  260 (* BV *);
  261 (* BVOP *);
  262 (* COP *);
  263 (* EDGCL *);
  264 (* EDG *);
  265 (* EDGOP *);
  266 (* EVCL *);
  267 (* EV *);
  268 (* EVOP *);
  269 (* FROMCL *);
  270 (* FROM *);
  271 (* FROMOP *);
  272 (* INFCL *);
  273 (* INF *);
  274 (* INFOP *);
  275 (* MOVOP *);
  276 (* MOVCL *);
  277 (* MOV *);
  278 (* NAME *);
  279 (* NODCL *);
  280 (* NOD *);
  281 (* NODOP *);
  282 (* NOWINNER *);
  283 (* PLCL *);
  284 (* PL *);
  285 (* PLOP *);
  286 (* PLANCL *);
  287 (* PLAN *);
  288 (* PLANOP *);
  289 (* RBD *);
  290 (* REG *);
  291 (* ROBBER *);
  292 (* SMELL *);
  293 (* VOTECL *);
  294 (* VOTE *);
  295 (* VOTEOP *);
  296 (* WINNER *);
  297 (* WORCL *);
  298 (* WOR *);
  299 (* WOROP *);
  300 (* WSKCL *);
  301 (* WSKOP *);
  302 (* DCCL *);
  303 (* DC *);
  304 (* DCOP *);
  305 (* SCCL *);
  306 (* SC *);
  307 (* SCOP *);
  308 (* FACCL *);
  309 (* FAC *);
  310 (* FACOP *);
  311 (* TURNCOAT *);
  312 (* STRAIGHTARROW *);
  313 (* CMOVOP *);
  314 (* CMOVCL *);
  315 (* ACCOP *);
  316 (* ACCCL *);
  317 (* ACC *);
  318 (* OFC *);
  319 (* OFCOP *);
  320 (* OFCCL *);
  321 (* NOTTELLING *);
  322 (* ACCUSED *);
    0|]

let yytransl_block = [|
  258 (* IDENT *);
    0|]

let yylhs = "\255\255\
\001\000\002\000\003\000\003\000\003\000\004\000\027\000\027\000\
\028\000\028\000\029\000\030\000\030\000\031\000\032\000\032\000\
\011\000\011\000\005\000\006\000\007\000\008\000\008\000\009\000\
\010\000\014\000\014\000\015\000\015\000\016\000\016\000\019\000\
\019\000\020\000\020\000\021\000\021\000\022\000\022\000\023\000\
\023\000\025\000\025\000\033\000\033\000\034\000\034\000\035\000\
\035\000\036\000\036\000\037\000\037\000\013\000\038\000\040\000\
\039\000\017\000\018\000\041\000\024\000\042\000\012\000\026\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000"

let yylen = "\002\000\
\003\000\021\000\043\000\001\000\002\000\005\000\003\000\000\000\
\004\000\000\000\005\000\002\000\002\000\011\000\004\000\000\000\
\002\000\005\000\005\000\005\000\005\000\003\000\002\000\005\000\
\005\000\004\000\000\000\007\000\000\000\006\000\000\000\004\000\
\000\000\005\000\000\000\006\000\000\000\005\000\000\000\005\000\
\000\000\006\000\000\000\008\000\000\000\007\000\000\000\004\000\
\000\000\005\000\000\000\005\000\000\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\002\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\063\000\065\000\000\000\000\000\
\066\000\004\000\000\000\000\000\067\000\000\000\068\000\000\000\
\069\000\000\000\070\000\000\000\071\000\000\000\000\000\072\000\
\000\000\073\000\000\000\074\000\000\000\000\000\075\000\054\000\
\000\000\000\000\000\000\005\000\064\000\000\000\000\000\000\000\
\000\000\023\000\000\000\000\000\000\000\000\000\017\000\001\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\022\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\058\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\006\000\000\000\019\000\000\000\020\000\000\000\
\021\000\000\000\024\000\000\000\025\000\000\000\018\000\000\000\
\000\000\000\000\000\000\048\000\000\000\000\000\015\000\000\000\
\059\000\000\000\000\000\000\000\050\000\052\000\000\000\000\000\
\062\000\000\000\000\000\000\000\000\000\000\000\000\000\046\000\
\000\000\000\000\000\000\044\000\000\000\000\000\000\000\000\000\
\026\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\055\000\000\000\000\000\032\000\000\000\057\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\028\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\056\000\000\000\000\000\034\000\
\000\000\000\000\002\000\000\000\000\000\030\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\036\000\000\000\
\000\000\000\000\000\000\060\000\000\000\000\000\000\000\000\000\
\038\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\040\000\061\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\003\000\042\000"

let yydgoto = "\012\000\
\014\000\017\000\021\000\023\000\025\000\027\000\029\000\032\000\
\034\000\036\000\039\000\015\000\041\000\125\000\139\000\161\000\
\075\000\114\000\136\000\158\000\181\000\193\000\204\000\213\000\
\218\000\046\000\000\000\000\000\000\000\000\000\000\000\072\000\
\061\000\063\000\065\000\068\000\070\000\147\000\152\000\174\000\
\197\000\122\000"

let yysindex = "\049\000\
\004\255\221\254\001\255\247\254\251\254\238\254\232\254\235\254\
\002\255\003\255\202\254\000\000\000\000\000\000\014\255\019\255\
\000\000\000\000\020\255\021\255\000\000\022\255\000\000\024\255\
\000\000\026\255\000\000\029\255\000\000\031\255\004\255\000\000\
\033\255\000\000\034\255\000\000\036\255\038\255\000\000\000\000\
\040\255\023\255\237\254\000\000\000\000\014\255\025\255\012\255\
\008\255\000\000\047\255\048\255\050\255\255\254\000\000\000\000\
\004\255\063\255\004\255\004\255\052\255\004\255\039\255\004\255\
\035\255\000\000\004\255\053\255\004\255\057\255\004\255\007\255\
\073\255\000\000\074\255\075\255\022\255\076\255\022\255\077\255\
\078\255\079\255\080\255\081\255\082\255\083\255\086\255\087\255\
\054\255\058\255\000\000\014\255\000\000\014\255\000\000\008\255\
\000\000\251\254\000\000\238\254\000\000\255\254\000\000\004\255\
\090\255\063\255\063\255\000\000\048\255\050\255\000\000\093\255\
\000\000\094\255\095\255\097\255\000\000\000\000\096\255\051\255\
\000\000\099\255\012\255\004\255\071\255\100\255\025\255\000\000\
\102\255\104\255\060\255\000\000\096\255\084\255\004\255\064\255\
\000\000\022\255\088\255\108\255\111\255\112\255\114\255\060\255\
\062\255\000\000\116\255\107\255\000\000\118\255\000\000\116\255\
\119\255\072\255\120\255\115\255\004\255\085\255\084\255\022\255\
\117\255\004\255\124\255\000\000\022\255\126\255\128\255\089\255\
\130\255\091\255\072\255\129\255\000\000\132\255\135\255\000\000\
\092\255\115\255\000\000\004\255\098\255\000\000\004\255\136\255\
\063\255\133\255\138\255\139\255\092\255\137\255\000\000\022\255\
\142\255\144\255\146\255\000\000\147\255\141\255\137\255\150\255\
\000\000\143\255\022\255\145\255\063\255\155\255\156\255\122\255\
\143\255\158\255\000\000\000\000\160\255\134\255\161\255\131\255\
\004\255\140\255\022\255\163\255\014\255\125\255\164\255\167\255\
\131\255\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\153\255\148\255\
\151\255\000\000\000\000\159\255\162\255\106\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\151\255\
\000\000\000\000\000\000\000\000\000\000\106\255\000\000\000\000\
\000\000\000\000\000\000\000\000\159\255\162\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\149\255\000\000\
\000\000\000\000\148\255\000\000\000\000\000\000\153\255\000\000\
\000\000\000\000\127\255\000\000\149\255\154\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\127\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\152\255\000\000\172\255\000\000\000\000\154\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\152\255\000\000\000\000\000\000\000\000\000\000\
\165\255\172\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\165\255\168\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\168\255\000\000\
\000\000\170\255\000\000\000\000\000\000\000\000\000\000\000\000\
\170\255\000\000\000\000\000\000\000\000\000\000\000\000\166\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\166\255\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\000\000\000\000\083\000\082\000\000\000\000\000\
\000\000\000\000\000\000\225\255\211\255\050\000\025\000\007\000\
\157\255\000\000\043\000\018\000\001\000\248\255\239\255\000\000\
\226\255\181\255\000\000\000\000\000\000\000\000\000\000\092\000\
\069\000\074\000\102\000\090\000\093\000\000\000\048\000\000\000\
\000\000\000\000"

let yytablesize = 217
let yytable = "\051\000\
\059\000\092\000\018\000\094\000\030\000\013\000\115\000\116\000\
\037\000\016\000\038\000\022\000\024\000\026\000\028\000\040\000\
\033\000\035\000\031\000\042\000\043\000\044\000\058\000\045\000\
\047\000\073\000\048\000\076\000\077\000\049\000\079\000\050\000\
\081\000\052\000\053\000\083\000\054\000\085\000\055\000\087\000\
\056\000\060\000\062\000\019\000\057\000\064\000\106\000\066\000\
\107\000\001\000\002\000\003\000\004\000\005\000\006\000\007\000\
\008\000\009\000\010\000\011\000\071\000\067\000\142\000\069\000\
\074\000\084\000\020\000\078\000\080\000\086\000\088\000\082\000\
\112\000\089\000\090\000\091\000\093\000\095\000\096\000\097\000\
\098\000\099\000\100\000\101\000\165\000\187\000\102\000\103\000\
\104\000\169\000\105\000\113\000\129\000\119\000\120\000\130\000\
\121\000\123\000\126\000\127\000\131\000\124\000\133\000\140\000\
\134\000\207\000\135\000\138\000\144\000\141\000\143\000\145\000\
\150\000\146\000\148\000\153\000\194\000\151\000\154\000\156\000\
\159\000\157\000\160\000\166\000\168\000\162\000\170\000\205\000\
\171\000\177\000\167\000\173\000\178\000\163\000\175\000\179\000\
\186\000\188\000\189\000\190\000\192\000\180\000\172\000\221\000\
\195\000\196\000\198\000\199\000\183\000\184\000\202\000\185\000\
\200\000\203\000\206\000\208\000\209\000\210\000\217\000\212\000\
\214\000\216\000\215\000\222\000\225\000\224\000\220\000\226\000\
\045\000\016\000\039\000\051\000\033\000\027\000\053\000\223\000\
\029\000\047\000\031\000\041\000\109\000\110\000\137\000\164\000\
\182\000\219\000\149\000\049\000\176\000\191\000\201\000\211\000\
\043\000\111\000\227\000\132\000\128\000\108\000\117\000\155\000\
\035\000\000\000\118\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\037\000"

let yycheck = "\031\000\
\046\000\077\000\002\001\079\000\026\001\002\001\106\000\107\000\
\063\001\045\001\065\001\021\001\018\001\032\001\039\001\002\001\
\015\001\015\001\040\001\001\001\001\001\001\001\042\001\002\001\
\001\001\057\000\001\001\059\000\060\000\001\001\062\000\001\001\
\064\000\001\001\001\001\067\000\001\001\069\000\001\001\071\000\
\001\001\017\001\031\001\043\001\022\001\038\001\092\000\001\001\
\094\000\001\000\002\000\003\000\004\000\005\000\006\000\007\000\
\008\000\009\000\010\000\011\000\062\001\014\001\138\000\014\001\
\002\001\013\001\066\001\016\001\030\001\013\001\064\001\037\001\
\104\000\001\001\001\001\001\001\001\001\001\001\001\001\001\001\
\001\001\001\001\001\001\001\001\160\000\185\000\001\001\001\001\
\035\001\165\000\033\001\002\001\124\000\001\001\001\001\025\001\
\002\001\001\001\048\001\001\001\001\001\006\001\001\001\135\000\
\001\001\205\000\047\001\024\001\001\001\046\001\023\001\001\001\
\051\001\002\001\001\001\009\001\192\000\002\001\001\001\001\001\
\001\001\050\001\008\001\007\001\001\001\157\000\001\001\203\000\
\001\001\001\001\162\000\002\001\001\001\049\001\044\001\001\001\
\001\001\005\001\001\001\001\001\004\001\050\001\054\001\219\000\
\003\001\002\001\001\001\001\001\180\000\052\001\001\001\183\000\
\012\001\011\001\010\001\001\001\001\001\036\001\028\001\002\001\
\001\001\001\001\029\001\001\001\001\001\041\001\027\001\001\001\
\016\001\064\001\003\001\013\001\046\001\025\001\013\001\221\000\
\023\001\030\001\007\001\010\001\098\000\100\000\133\000\159\000\
\178\000\217\000\144\000\037\001\171\000\189\000\199\000\209\000\
\027\001\102\000\225\000\127\000\123\000\096\000\109\000\152\000\
\049\001\255\255\110\000\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\052\001"

let yynames_const = "\
  EOF\000\
  EOL\000\
  BVCL\000\
  BV\000\
  BVOP\000\
  COP\000\
  EDGCL\000\
  EDG\000\
  EDGOP\000\
  EVCL\000\
  EV\000\
  EVOP\000\
  FROMCL\000\
  FROM\000\
  FROMOP\000\
  INFCL\000\
  INF\000\
  INFOP\000\
  MOVOP\000\
  MOVCL\000\
  MOV\000\
  NAME\000\
  NODCL\000\
  NOD\000\
  NODOP\000\
  NOWINNER\000\
  PLCL\000\
  PL\000\
  PLOP\000\
  PLANCL\000\
  PLAN\000\
  PLANOP\000\
  RBD\000\
  REG\000\
  ROBBER\000\
  SMELL\000\
  VOTECL\000\
  VOTE\000\
  VOTEOP\000\
  WINNER\000\
  WORCL\000\
  WOR\000\
  WOROP\000\
  WSKCL\000\
  WSKOP\000\
  DCCL\000\
  DC\000\
  DCOP\000\
  SCCL\000\
  SC\000\
  SCOP\000\
  FACCL\000\
  FAC\000\
  FACOP\000\
  TURNCOAT\000\
  STRAIGHTARROW\000\
  CMOVOP\000\
  CMOVCL\000\
  ACCOP\000\
  ACCCL\000\
  ACC\000\
  OFC\000\
  OFCOP\000\
  OFCCL\000\
  NOTTELLING\000\
  ACCUSED\000\
  "

let yynames_block = "\
  IDENT\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'bot) in
    let _2 = (peek_val parser_env 1 : 'ptype) in
    Obj.repr(
# 129 "parser.mly"
                   ( {reg_bot = _1; reg_ptype = _2} )
# 466 "parser.ml"
               : Message.register))
; (fun parser_env ->
    let _4 = (peek_val parser_env 17 : 'bot) in
    let _7 = (peek_val parser_env 14 : 'bot) in
    let _9 = (peek_val parser_env 12 : 'cop_list) in
    let _12 = (peek_val parser_env 9 : 'nod_list) in
    let _17 = (peek_val parser_env 4 : 'edg_list) in
    Obj.repr(
# 140 "parser.mly"
      ( { wsk_name = _4;
          wsk_robber = _7;
          wsk_cops = _9;
          wsk_nods = _12;
          wsk_edgs = _17;
        }
      )
# 483 "parser.ml"
               : Message.world_skeleton))
; (fun parser_env ->
    let _4 = (peek_val parser_env 39 : 'world) in
    let _7 = (peek_val parser_env 36 : 'loot) in
    let _11 = (peek_val parser_env 32 : 'dc_list) in
    let _16 = (peek_val parser_env 27 : 'sc_list) in
    let _21 = (peek_val parser_env 22 : 'fac_list) in
    let _26 = (peek_val parser_env 17 : 'bv_list) in
    let _31 = (peek_val parser_env 12 : 'ev_list) in
    let _35 = (peek_val parser_env 8 : 'distance) in
    let _39 = (peek_val parser_env 4 : 'pl_list) in
    Obj.repr(
# 161 "parser.mly"
      ( World {
          wor_wor = _4;
          wor_rbd = _7;
	  wor_dc  = _11;
	  wor_sc  = _16;
	  wor_fac = _21;
          wor_bvs = _26;
          wor_evs = _31;
          wor_smell = _35;
          wor_pls = _39;
        }
      )
# 509 "parser.ml"
               : Message.world_or_game_over))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 173 "parser.mly"
          ( mk_game_over _1 )
# 516 "parser.ml"
               : Message.world_or_game_over))
; (fun parser_env ->
    Obj.repr(
# 174 "parser.mly"
                ( Game_over )
# 522 "parser.ml"
               : Message.world_or_game_over))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'loc) in
    let _3 = (peek_val parser_env 2 : 'ptype) in
    let _4 = (peek_val parser_env 1 : 'bot) in
    Obj.repr(
# 178 "parser.mly"
                          ( { mov_loc = _2; mov_ptype = _3; mov_bot = _4 } )
# 531 "parser.ml"
               : Message.move))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : Message.move) in
    let _3 = (peek_val parser_env 0 : 'mov_list) in
    Obj.repr(
# 182 "parser.mly"
                      ( _1 :: _3 )
# 539 "parser.ml"
               : 'mov_list))
; (fun parser_env ->
    Obj.repr(
# 183 "parser.mly"
                      ( [] )
# 545 "parser.ml"
               : 'mov_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'bot) in
    let _4 = (peek_val parser_env 0 : 'acc_list) in
    Obj.repr(
# 187 "parser.mly"
                         ( _2 :: _4 )
# 553 "parser.ml"
               : 'acc_list))
; (fun parser_env ->
    Obj.repr(
# 188 "parser.mly"
                      ( [] )
# 559 "parser.ml"
               : 'acc_list))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'mov_list) in
    Obj.repr(
# 192 "parser.mly"
                                 ( _3 )
# 566 "parser.ml"
               : 'moves))
; (fun parser_env ->
    Obj.repr(
# 196 "parser.mly"
                 ( Turncoat )
# 572 "parser.ml"
               : 'offer))
; (fun parser_env ->
    Obj.repr(
# 197 "parser.mly"
                      ( Straight )
# 578 "parser.ml"
               : 'offer))
; (fun parser_env ->
    let _3 = (peek_val parser_env 8 : 'offer) in
    let _4 = (peek_val parser_env 7 : 'moves) in
    let _7 = (peek_val parser_env 4 : 'acc_list) in
    Obj.repr(
# 205 "parser.mly"
                 ( { mov_offer = _3; mov_moves = _4; mov_accuse = _7 } )
# 587 "parser.ml"
               : 'clean_cop_move))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'bot) in
    let _4 = (peek_val parser_env 0 : 'ofc_list) in
    Obj.repr(
# 209 "parser.mly"
                         ( _2 :: _4 )
# 595 "parser.ml"
               : 'ofc_list))
; (fun parser_env ->
    Obj.repr(
# 210 "parser.mly"
                      ( [] )
# 601 "parser.ml"
               : 'ofc_list))
; (fun parser_env ->
    Obj.repr(
# 214 "parser.mly"
                   ( NotTelling )
# 607 "parser.ml"
               : Message.offered_cops))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'ofc_list) in
    Obj.repr(
# 215 "parser.mly"
                                 ( Offered _3 )
# 614 "parser.ml"
               : Message.offered_cops))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'inf_list) in
    Obj.repr(
# 219 "parser.mly"
                                 ( _3 )
# 621 "parser.ml"
               : Message.inf list))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'plan_list) in
    Obj.repr(
# 223 "parser.mly"
                                    ( _3 )
# 628 "parser.ml"
               : Message.plan list))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'vote_list) in
    Obj.repr(
# 227 "parser.mly"
                                    ( _3 )
# 635 "parser.ml"
               : Message.vote list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'bot) in
    Obj.repr(
# 231 "parser.mly"
                    ( Winner _2 )
# 642 "parser.ml"
               : Message.vote_res))
; (fun parser_env ->
    Obj.repr(
# 232 "parser.mly"
                    ( Nowinner )
# 648 "parser.ml"
               : Message.vote_res))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'from_inform_list) in
    Obj.repr(
# 236 "parser.mly"
                                            ( _3 )
# 655 "parser.ml"
               : (string * Message.inf list) list))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'from_plan_list) in
    Obj.repr(
# 239 "parser.mly"
                                            ( _3 )
# 662 "parser.ml"
               : (string * Message.plan list) list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'bot) in
    let _4 = (peek_val parser_env 0 : 'cop_list) in
    Obj.repr(
# 244 "parser.mly"
                          ( _2 :: _4 )
# 670 "parser.ml"
               : 'cop_list))
; (fun parser_env ->
    Obj.repr(
# 245 "parser.mly"
                      ( [] )
# 676 "parser.ml"
               : 'cop_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 5 : 'loc) in
    let _3 = (peek_val parser_env 4 : 'node_tag) in
    let _4 = (peek_val parser_env 3 : 'coordinate) in
    let _5 = (peek_val parser_env 2 : 'coordinate) in
    let _7 = (peek_val parser_env 0 : 'nod_list) in
    Obj.repr(
# 250 "parser.mly"
      ( 
      let n = 
        { nod_num = !nnodes; 
          nod_loc = _2; nod_tag = _3; nod_x = _4; nod_y = _5 } 
      in
      incr nnodes;
      n :: _7 )
# 693 "parser.ml"
               : 'nod_list))
; (fun parser_env ->
    Obj.repr(
# 257 "parser.mly"
                      ( [] )
# 699 "parser.ml"
               : 'nod_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 4 : 'loc) in
    let _3 = (peek_val parser_env 3 : 'loc) in
    let _4 = (peek_val parser_env 2 : 'edge_type) in
    let _6 = (peek_val parser_env 0 : 'edg_list) in
    Obj.repr(
# 262 "parser.mly"
      ( { edg_src = _2; edg_dst = _3; edg_type = _4 } :: _6 )
# 709 "parser.ml"
               : 'edg_list))
; (fun parser_env ->
    Obj.repr(
# 263 "parser.mly"
                      ( [] )
# 715 "parser.ml"
               : 'edg_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'bot) in
    let _4 = (peek_val parser_env 0 : 'dc_list) in
    Obj.repr(
# 267 "parser.mly"
                        ( _2 :: _4 )
# 723 "parser.ml"
               : 'dc_list))
; (fun parser_env ->
    Obj.repr(
# 268 "parser.mly"
                      ( [] )
# 729 "parser.ml"
               : 'dc_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'bot) in
    let _3 = (peek_val parser_env 2 : 'bot) in
    let _5 = (peek_val parser_env 0 : 'sc_list) in
    Obj.repr(
# 272 "parser.mly"
                            ( { sc_brain = _2; sc_cop = _3 } :: _5 )
# 738 "parser.ml"
               : 'sc_list))
; (fun parser_env ->
    Obj.repr(
# 273 "parser.mly"
                      ( [] )
# 744 "parser.ml"
               : 'sc_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 4 : 'bot) in
    let _3 = (peek_val parser_env 3 : 'bot) in
    let _4 = (peek_val parser_env 2 : 'world) in
    let _6 = (peek_val parser_env 0 : 'fac_list) in
    Obj.repr(
# 277 "parser.mly"
                                   ( { fac_accusing = _2; fac_accused = _3;
				       fac_wor = _4 } :: _6 )
# 755 "parser.ml"
               : 'fac_list))
; (fun parser_env ->
    Obj.repr(
# 279 "parser.mly"
                      ( [] )
# 761 "parser.ml"
               : 'fac_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'loc) in
    let _3 = (peek_val parser_env 2 : 'bank_value) in
    let _5 = (peek_val parser_env 0 : 'bv_list) in
    Obj.repr(
# 283 "parser.mly"
                                   ( { bv_loc = _2; bv_value = _3 } :: _5 )
# 770 "parser.ml"
               : 'bv_list))
; (fun parser_env ->
    Obj.repr(
# 284 "parser.mly"
                      ( [] )
# 776 "parser.ml"
               : 'bv_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'loc) in
    let _3 = (peek_val parser_env 2 : 'world) in
    let _5 = (peek_val parser_env 0 : 'ev_list) in
    Obj.repr(
# 289 "parser.mly"
                             ( { ev_loc = _2;  ev_world = _3 } :: _5 )
# 785 "parser.ml"
               : 'ev_list))
; (fun parser_env ->
    Obj.repr(
# 290 "parser.mly"
                      ( [] )
# 791 "parser.ml"
               : 'ev_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 4 : 'bot) in
    let _3 = (peek_val parser_env 3 : 'loc) in
    let _4 = (peek_val parser_env 2 : 'ptype) in
    let _6 = (peek_val parser_env 0 : 'pl_list) in
    Obj.repr(
# 295 "parser.mly"
      ( { pl_bot = _2; pl_loc = _3; pl_ptype = _4 } :: _6 )
# 801 "parser.ml"
               : 'pl_list))
; (fun parser_env ->
    Obj.repr(
# 296 "parser.mly"
                      ( [] )
# 807 "parser.ml"
               : 'pl_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 6 : 'bot) in
    let _3 = (peek_val parser_env 5 : 'loc) in
    let _4 = (peek_val parser_env 4 : 'ptype) in
    let _5 = (peek_val parser_env 3 : 'world) in
    let _6 = (peek_val parser_env 2 : 'certainty) in
    let _8 = (peek_val parser_env 0 : 'inf_list) in
    Obj.repr(
# 301 "parser.mly"
      ( { inf_bot = _2; inf_loc = _3; inf_ptype = _4; inf_world = _5;
          inf_certainty = _6 } :: _8 )
# 820 "parser.ml"
               : 'inf_list))
; (fun parser_env ->
    Obj.repr(
# 303 "parser.mly"
                      ( [] )
# 826 "parser.ml"
               : 'inf_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 5 : 'bot) in
    let _3 = (peek_val parser_env 4 : 'loc) in
    let _4 = (peek_val parser_env 3 : 'ptype) in
    let _5 = (peek_val parser_env 2 : 'world) in
    let _7 = (peek_val parser_env 0 : 'plan_list) in
    Obj.repr(
# 308 "parser.mly"
      ( { plan_bot = _2; plan_loc = _3; plan_ptype = _4; plan_world = _5 }
        :: _7 )
# 838 "parser.ml"
               : 'plan_list))
; (fun parser_env ->
    Obj.repr(
# 310 "parser.mly"
                      ( [] )
# 844 "parser.ml"
               : 'plan_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'bot) in
    let _4 = (peek_val parser_env 0 : 'vote_list) in
    Obj.repr(
# 314 "parser.mly"
                            ( _2 :: _4 )
# 852 "parser.ml"
               : 'vote_list))
; (fun parser_env ->
    Obj.repr(
# 315 "parser.mly"
                      ( [] )
# 858 "parser.ml"
               : 'vote_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'bot) in
    let _4 = (peek_val parser_env 1 : Message.inf list) in
    let _5 = (peek_val parser_env 0 : 'from_inform_list) in
    Obj.repr(
# 320 "parser.mly"
      ( (_2, _4) :: _5 )
# 867 "parser.ml"
               : 'from_inform_list))
; (fun parser_env ->
    Obj.repr(
# 321 "parser.mly"
                      ( [] )
# 873 "parser.ml"
               : 'from_inform_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'bot) in
    let _4 = (peek_val parser_env 1 : Message.plan list) in
    let _5 = (peek_val parser_env 0 : 'from_plan_list) in
    Obj.repr(
# 326 "parser.mly"
      ( (_2, _4) :: _5 )
# 882 "parser.ml"
               : 'from_plan_list))
; (fun parser_env ->
    Obj.repr(
# 327 "parser.mly"
                      ( [] )
# 888 "parser.ml"
               : 'from_plan_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 333 "parser.mly"
          ( mk_ptype _1 )
# 895 "parser.ml"
               : 'ptype))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 337 "parser.mly"
           ( mk_node_tag _1 )
# 902 "parser.ml"
               : 'node_tag))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 341 "parser.mly"
           ( mk_edge_type _1 )
# 909 "parser.ml"
               : 'edge_type))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 347 "parser.mly"
           ( mk_coordinate _1 )
# 916 "parser.ml"
               : 'coordinate))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 351 "parser.mly"
          ( mk_world _1 )
# 923 "parser.ml"
               : 'world))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 355 "parser.mly"
          ( mk_loot _1 )
# 930 "parser.ml"
               : 'loot))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 359 "parser.mly"
          ( mk_bank_value _1 )
# 937 "parser.ml"
               : 'bank_value))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 363 "parser.mly"
          ( mk_distance _1 )
# 944 "parser.ml"
               : 'distance))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 367 "parser.mly"
          ( mk_certainty _1 )
# 951 "parser.ml"
               : 'certainty))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 373 "parser.mly"
          ( _1 )
# 958 "parser.ml"
               : 'bot))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 377 "parser.mly"
           ( _1 )
# 965 "parser.ml"
               : 'loc))
(* Entry register *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry world_skeleton *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry world_or_game_over *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry move *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry inform *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry plan *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry vote *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry vote_res *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry from_inform *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry from_plan *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry offered_cops *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
|]
let yytables =
  { actions=yyact;
    transl_const=yytransl_const;
    transl_block=yytransl_block;
    lhs=yylhs;
    len=yylen;
    defred=yydefred;
    dgoto=yydgoto;
    sindex=yysindex;
    rindex=yyrindex;
    gindex=yygindex;
    tablesize=yytablesize;
    table=yytable;
    check=yycheck;
    error_function=parse_error;
    names_const=yynames_const;
    names_block=yynames_block }
let register (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 1 lexfun lexbuf : Message.register)
let world_skeleton (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 2 lexfun lexbuf : Message.world_skeleton)
let world_or_game_over (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 3 lexfun lexbuf : Message.world_or_game_over)
let move (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 4 lexfun lexbuf : Message.move)
let inform (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 5 lexfun lexbuf : Message.inf list)
let plan (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 6 lexfun lexbuf : Message.plan list)
let vote (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 7 lexfun lexbuf : Message.vote list)
let vote_res (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 8 lexfun lexbuf : Message.vote_res)
let from_inform (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 9 lexfun lexbuf : (string * Message.inf list) list)
let from_plan (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 10 lexfun lexbuf : (string * Message.plan list) list)
let offered_cops (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 11 lexfun lexbuf : Message.offered_cops)
