/* $Id: parser.mly,v 1.6.4.3 2005/07/10 00:08:55 lefessan Exp $ */

%{

open Message;;

let mk_ptype = function
  | "cop-foot" -> Cop_foot
  | "cop-car" -> Cop_car
  | "robber" -> Robber
  | _ -> raise Parse_error
;;

let mk_node_tag = function
  | "hq" -> Hq
  | "bank" -> Bank
  | "robber-start" -> Robber_start
  | "ordinary" -> Ordinary
  | _ -> raise Parse_error
;;

let mk_edge_type = function
  | "car" -> Car
  | "foot" -> Foot
  | _ -> raise Parse_error
;;

let mk_game_over = function
  | "game-over" -> Game_over
  | _ -> raise Parse_error
;;

let between lower upper s =
  try
    let n = int_of_string s in
    if n < lower || n > upper then raise Parse_error;
    n
  with Failure _ -> raise Parse_error
;;

let mk_coordinate s = between 0 1023 s;;
let mk_world s = between 0 200 s;;
let mk_loot s = between 0 6000 s;;
let mk_bank_value s = between 0 1000 s;;
let mk_distance s = between 0 1_000_000_000 s;;
let mk_certainty s = between (-100) 100 s;;

%}

%token EOF
%token EOL
%token <string> IDENT
%token BVCL
%token BV
%token BVOP
%token COP
%token EDGCL
%token EDG
%token EDGOP
%token EVCL
%token EV
%token EVOP
%token FROMCL
%token FROM
%token FROMOP
%token INFCL
%token INF
%token INFOP
%token MOVOP MOVCL MOV
%token NAME
%token NODCL
%token NOD
%token NODOP
%token NOWINNER
%token PLCL
%token PL
%token PLOP
%token PLANCL
%token PLAN
%token PLANOP
%token RBD
%token REG
%token ROBBER
%token SMELL
%token VOTECL
%token VOTE
%token VOTEOP
%token WINNER
%token WORCL
%token WOR
%token WOROP
%token WSKCL
%token WSKOP
%token DCCL DC DCOP
%token SCCL SC SCOP
%token FACCL FAC FACOP

%token TURNCOAT STRAIGHTARROW
%token CMOVOP CMOVCL ACCOP ACCCL ACC
%token OFC OFCOP OFCCL
%token NOTTELLING ACCUSED

%start register
%type <Message.register> register
%start world_skeleton
%type <Message.world_skeleton> world_skeleton
%start world_or_game_over
%type <Message.world_or_game_over> world_or_game_over
%start move
%type <Message.move> move
%start inform
%type <Message.inf list> inform
%start plan
%type <Message.plan list> plan
%start vote
%type <Message.vote list> vote
%start vote_res
%type <Message.vote_res> vote_res
%start from_inform
%type <(string * Message.inf list) list> from_inform
%start from_plan
%type <(string * Message.plan list) list> from_plan
%start offered_cops
%type <Message.offered_cops> offered_cops

%%

register:
  | bot ptype EOL  { {reg_bot = $1; reg_ptype = $2} }
;

world_skeleton:
  | WSKOP EOL
    NAME bot EOL
    ROBBER bot EOL
    cop_list
    NODOP EOL nod_list NODCL EOL
    EDGOP EOL edg_list EDGCL EOL
    WSKCL EOL
      { { wsk_name = $4;
          wsk_robber = $7;
          wsk_cops = $9;
          wsk_nods = $12;
          wsk_edgs = $17;
        }
      }
;

world_or_game_over:
  | WOROP EOL
    WOR world EOL
    RBD loot EOL
    DCOP EOL dc_list DCCL EOL
    SCOP EOL sc_list SCCL EOL
    FACOP EOL fac_list FACCL EOL
    BVOP EOL bv_list BVCL EOL
    EVOP EOL ev_list EVCL EOL
    SMELL distance EOL
    PLOP EOL pl_list PLCL EOL
    WORCL EOL
      { World {
          wor_wor = $4;
          wor_rbd = $7;
	  wor_dc  = $11;
	  wor_sc  = $16;
	  wor_fac = $21;
          wor_bvs = $26;
          wor_evs = $31;
          wor_smell = $35;
          wor_pls = $39;
        }
      }
  | IDENT { mk_game_over $1 }
  | ACCUSED EOL { Game_over }
;

move:
  | MOV loc ptype bot EOL { { mov_loc = $2; mov_ptype = $3; mov_bot = $4 } }
;

mov_list:
  | move EOL mov_list { $1 :: $3 }
  | /* empty */       { [] }
;

acc_list:
  | ACC bot EOL acc_list { $2 :: $4 }
  | /* empty */       { [] }
;

moves:
  | MOVOP EOL mov_list MOVCL EOL { $3 }
;

offer:
  | TURNCOAT EOL { Turncoat }
  | STRAIGHTARROW EOL { Straight }
;

clean_cop_move:
  | CMOVOP EOL
      offer
      moves
      ACCOP EOL acc_list ACCCL EOL
      CMOVCL EOL { { mov_offer = $3; mov_moves = $4; mov_accuse = $7 } }
;

ofc_list:
  | OFC bot EOL ofc_list { $2 :: $4 }
  | /* empty */       { [] }
;

offered_cops:
  | NOTTELLING EOL { NotTelling }
  | OFCOP EOL ofc_list OFCCL EOL { Offered $3 }
;

inform:
  | INFOP EOL inf_list INFCL EOL { $3 }
;

plan:
  | PLANOP EOL plan_list PLANCL EOL { $3 }
;

vote:
  | VOTEOP EOL vote_list VOTECL EOL { $3 }
;

vote_res:
  | WINNER bot EOL  { Winner $2 }
  | NOWINNER EOL    { Nowinner }
;;

from_inform:
  | FROMOP EOL from_inform_list FROMCL EOL  { $3 }

from_plan:
  | FROMOP EOL from_plan_list FROMCL EOL    { $3 }

/* ---- */

cop_list:
  | COP bot EOL cop_list  { $2 :: $4 }
  | /* empty */       { [] }
;

nod_list:
  | NOD loc node_tag coordinate coordinate EOL nod_list
      { 
      let n = 
        { nod_num = !nnodes; 
          nod_loc = $2; nod_tag = $3; nod_x = $4; nod_y = $5 } 
      in
      incr nnodes;
      n :: $7 }
  | /* empty */       { [] }
;

edg_list:
  | EDG loc loc edge_type EOL edg_list
      { { edg_src = $2; edg_dst = $3; edg_type = $4 } :: $6 }
  | /* empty */       { [] }
;

dc_list:
  | DC bot EOL dc_list  { $2 :: $4 }
  | /* empty */       { [] }
;

sc_list:
  | SC bot bot EOL sc_list  { { sc_brain = $2; sc_cop = $3 } :: $5 }
  | /* empty */       { [] }
;

fac_list:
  | SC bot bot world EOL fac_list  { { fac_accusing = $2; fac_accused = $3;
				       fac_wor = $4 } :: $6 }
  | /* empty */       { [] }
;

bv_list:
  | BV loc bank_value EOL bv_list  { { bv_loc = $2; bv_value = $3 } :: $5 }
  | /* empty */       { [] }
;


ev_list:
  | EV loc world EOL ev_list { { ev_loc = $2;  ev_world = $3 } :: $5 }
  | /* empty */       { [] }
;

pl_list:
  | PL bot loc ptype EOL pl_list
      { { pl_bot = $2; pl_loc = $3; pl_ptype = $4 } :: $6 }
  | /* empty */       { [] }
;

inf_list:
  | INF bot loc ptype world certainty EOL inf_list
      { { inf_bot = $2; inf_loc = $3; inf_ptype = $4; inf_world = $5;
          inf_certainty = $6 } :: $8 }
  | /* empty */       { [] }
;

plan_list:
  | PLAN bot loc ptype world EOL plan_list
      { { plan_bot = $2; plan_loc = $3; plan_ptype = $4; plan_world = $5 }
        :: $7 }
  | /* empty */       { [] }
;

vote_list:
  | VOTE bot EOL vote_list  { $2 :: $4 }
  | /* empty */       { [] }
;

from_inform_list:
  | FROM bot EOL inform from_inform_list
      { ($2, $4) :: $5 }
  | /* empty */       { [] }
;

from_plan_list:
  | FROM bot EOL plan from_plan_list
      { ($2, $4) :: $5 }
  | /* empty */       { [] }
;

/* ---- types concrets */

ptype:
  | IDENT { mk_ptype $1 }
;

node_tag:
  | IDENT  { mk_node_tag $1 }
;

edge_type:
  | IDENT  { mk_edge_type $1 }
;

/* ---- entiers */

coordinate:
  | IDENT  { mk_coordinate $1 }
;

world:
  | IDENT { mk_world $1 }
;

loot:
  | IDENT { mk_loot $1 }
;

bank_value:
  | IDENT { mk_bank_value $1 }
;

distance :
  | IDENT { mk_distance $1 }
;

certainty:
  | IDENT { mk_certainty $1 }
;

/* ---- idents */

bot:
  | IDENT { $1 }
;

loc:
  | IDENT  { $1 }
;
