open Topology
open Message
open Info

let mk_plan info cop_name typ nod =
  { plan_bot = cop_name;
    plan_loc = nod.Message.nod_loc;
    plan_ptype = typ;
    plan_world = Info.plan_for_world info }

let mk_plans info movs =
  let names = (topo info).cops_name in
  List.map (fun (i,(nod,typ)) ->
	      mk_plan info names.(i) typ nod) movs

let change_cops_move_type info =
  let topo = topo info in

  let on_car i = let _,ptype = get_cop_i info i in ptype = Cop_car
  and on_hq i = let nod,_ = get_cop_i info i in nod.nod_tag = Hq in

  let ncars = ref 0 in
  for i = 0 to topo.ncops - 1 do if on_car i then incr ncars done;

  let rec change_to_car accu = function
    | [] -> accu
    | n :: q when on_car n || not (on_hq n) -> change_to_car accu q
    | n :: q ->
	let accu = (n, Cop_car) :: accu in
	incr ncars;
	if (!ncars < 2) then change_to_car accu q else accu in
  let rec change_to_foot accu = function
    | [] -> accu
    | n :: q when not (on_car n) || not (on_hq n) -> change_to_foot accu q
    | n :: q ->
	let accy = (n, Cop_foot) :: accu in
	decr ncars;
	if (!ncars > 2) then change_to_foot accu q else accu in

  let cops = (Info.get_my_cops info) @ (Info.get_other_cops info) in
  (* On dplace en priorit nos flics (ils obissent, au moins, eux). *)

  if (!ncars > 2) then change_to_foot [] cops
  else if (!ncars < 2) then change_to_car [] cops
  else []


(*
(* Currently unused *)
let random_neigh info cop_name =
  let w,t = Info.get_cop info cop_name in
  let d = possible_dest (Info.topo info) w t in
  let l = NS.elements d in
  List.nth l (Random.int (List.length l))

(* Currently unused *)
let random_plan info =
  let topo = Info.topo info in
  List.map 
    (fun (cop_name,_) ->
       let (_,typ) = Info.get_cop info cop_name in
       let dest = random_neigh info cop_name in
       mk_plan info cop_name typ dest
    ) topo.cops_number


(* Currently unused *)
let optimizer_plan info f =
  let topo = Info.topo info in
  List.map
    (fun (cop_name,i) ->
       let w,t = Info.get_cop_i info i in
       let d = possible_dest topo w t in
       let max_r,dest = NS.fold (fun n ((max_r,max_n) as accu) -> 
			       let r = f i t w n in
			       if r > neg_infinity then (r,n) else accu
			    ) d (neg_infinity,w) in
       assert (max_r <> neg_infinity);
       mk_plan info cop_name t dest
    ) topo.cops_number

let global_optimizer_plan info f =
  let topo = Info.topo info in
  let rec moves accu m = function
    | [] -> 
	let m = Array.of_list (List.rev m) in
	let score = f m in
	let (max_score,max_plan) = accu in
	if score > max_score then (score,m) else accu
    | (_,i)::cops ->
	let n,t = Info.get_cop_i info i in
	let dests = possible_dest topo n t in
	NS.fold (fun n' accu -> moves accu ((t,n,n')::m) cops) dests accu in
  let (score,m) = moves (neg_infinity,[||]) [] topo.cops_number in
  assert (m != [||]);
  let m = Array.to_list m in
  List.map2 (fun (cop_name,_) (typ,_,dest) -> mk_plan info cop_name typ dest
	    ) topo.cops_number m

(* Currently unused *)
let repulser_plan info =
  let topo = Info.topo info in
  global_optimizer_plan info
    (fun moves ->
       let s = ref 0. in
       for i = 0 to topo.ncops - 1 do 
	 let _,_,ni = moves.(i) in
	 for j = 0 to topo.ncops - 1 do
	   if (i != j) then
	     let _,_,nj = moves.(j) in
	     let d = dist_f topo ni nj in
	     s := !s -. 1. /. (float_of_int d)
	 done
       done;
       !s
    )

(* Currently unused *)
let plan_of info plans cop_name =
  let p = List.find (fun p -> p.plan_bot = cop_name) plans in
  p.plan_ptype, (get_node (Info.topo info) (p.plan_loc))

(* Currently unused *)
let my_plan info plans =
  plan_of info plans ((Info.topo info).name)

*)


(*
let damien_plan info =
  let topo = Info.topo info in
  let choose_move (nod,ptype) set =
    let best = ref [] in
    let best_dist = ref 10000 in
    let f nod =
      let d = dist_to_set topo nod set in
      if d = !best_dist then
        best := nod :: !best
      else if d < !best_dist then begin
        best_dist := d;
        best := [nod];
      end;
    in
    NS.iter f (possible_dest topo nod ptype);
    let i = Random.int (List.length !best) in
    List.nth !best i
  in
  let targets = ref (Info.get_possible info) in
  let (_,my_type) as me = Info.get_me info in
  let my_move = choose_move me !targets in
  targets := NS.remove my_move !targets;
  let f cop_name cop_i loc ptype =
    if cop_name = topo.name then 
      mk_plan info cop_name ptype my_move
    else begin
      let move = choose_move (loc,ptype) !targets in
      targets := NS.remove move !targets;
      mk_plan info cop_name ptype move
    end
  in
  (my_type,my_move),
  Info.map_cops info f
*)

(* S'assurer d'attraper le voleur si on peut. On suppose que possible
   a un seul element. *)
let catch_move info possible nod typ =
  let topo = topo info in
  let dests = possible_dest topo nod typ in
  if NS.mem possible dests
  then Some (typ,possible)
  else None

type movs = (int * (ptype * nod)) list

let improve_plan info plan =
  (* Pour chacun de mes policiers, s'il peut attraper, le faire *)
  let possible = get_possible info in
  if NS.cardinal possible = 1 then
    let possible = NS.choose possible in
    let my_cops = Info.get_my_cops info in
    List.map
      (fun (i,(typ,nod)) ->
	 i,
	 if List.mem i my_cops then
	   let (nod0,typ0) = Info.get_cop_i info i in
	   match catch_move info possible nod0 typ0 with
	     | Some (typ',nod') -> (typ',nod')
	     | None -> (typ,nod)
	 else (typ,nod))
      plan
  else plan

let degrade_plan info plan =
  match Info.get_robber info with
    | Some robber ->
	List.map
	  (fun (i,(typ,nod)) ->
	     i,
	     if nod=robber then
	       let (nod0,typ0) = Info.get_cop_i info i in
	       (typ0,nod0) (* forcer  rester sur place *)
	     else (typ,nod))
	  plan
    | None -> plan

