(* $Id: robber13.ml,v 1.1.2.2 2005/07/10 12:07:17 lefessan Exp $ *)

open Args
open Message
open Robber_types
open Robber_topology
open Robber
  
  
let compute_node_value t w valuate_path i =
  let nod = t.nodes.(i) in
  
  let arrested_risk = 
    let risk = ref 0 in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car ->
          if t.car_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
(*              Printf.fprintf stderr "Arrested risk by car cop in %d\n" i; *)
              incr risk
            end
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
(*              Printf.fprintf stderr "Arrested risk by foot cop in %d\n" i; *)
              incr risk
            end
      | Robber -> assert false
    done;
    !risk
  in
  
  let smelled_risk = 
    let risk = ref (2 * arrested_risk) in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car -> ()
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) = 2 then
            incr risk 
      | Robber -> assert false
    done;
    if !risk > 0 then 1 else 0
  in

  let paths = valuate_path i in
  
  ( arrested_risk, smelled_risk, -paths )
  
      
      
let inform t w = 
  let robbernode = t.nodes.(t.robber_node) in  
  let valuate_path = 
    Robber_misc.sum_to_cops_from t w  [t.robber_node] in
  let valuate_path = 
    Robber_misc.path_from t w 
      (fun len i -> 
        let node = t.nodes.(i) in
        let total = (1+len) * node.node_copsum in
        let total = total - 5 * node.node_danger in
        let total = 
          if robbernode.node_copdist > 3 then total
          else
            total + 3 * node.node_density in
        let total = total + 5 * Robber_draw.score_bank t w node in
        let total = if i = t.last_robber_node then
            total - 50
          else total in 
(* DIFF robber9: try to escape from possibles *)
        let total = if w.state_possibles.(i) then total else total + 5 in
        total
    
    ) (fun len i -> 
        let node = t.nodes.(i) in
        let total = (3*len) * node.node_copsum in
        let total = total - 10 * node.node_danger in
        let total = 
          if robbernode.node_copdist > 3 then total
          else
          total + 3 * node.node_density in
        let total = total + Robber_draw.score_bank t w node in
        let total = if i = t.last_robber_node then
            total - 50
          else total in 
        total
    ) 7 [t.robber_node]  in
  
  let neighbours = t.foot_edges.(t.robber_node) in 
  
  let (all_cops, foot_cops, car_cops) = cops_proximities t w t.robber_node in
  
  let list = ref [] in
  for i = 0 to t.ncops - 1 do
    match w.cops_ptypes.(i) with
      Cop_car ->
        list := (
          t.car_dist.(w.cops_nodes.(i)).(t.robber_node), 
          "car",
          t.cops_names.(i), 
          w.cops_nodes.(i)
       ) :: !list
    | Cop_foot ->
        list := (t.foot_dist.(w.cops_nodes.(i)).(t.robber_node), "foot",
          t.cops_names.(i),
          w.cops_nodes.(i)          
        ) :: !list
    | _ -> ()
  done;
  let list = List.sort compare !list in
  Printf.fprintf stderr "COPS: \n";
  List.iter (fun (dist, ptype, name, pos) ->
      Printf.fprintf stderr " %d (%s)" dist ptype
  ) list;
  Printf.fprintf stderr "\n";
  
  let neighbours = List.map (fun i ->
        compute_node_value t w valuate_path i , i        
    ) neighbours in
  
  let neighbours = List.sort compare neighbours in
  let next_node = snd (List.hd neighbours) in
  t.proposed_robber_node <- next_node;
  
  Printf.fprintf stderr "\nCurrent position: %d\n" t.robber_node;
  Printf.fprintf stderr "Move sorted:\n";
  List.iter (fun ( (a,b,c), num) ->
      Printf.fprintf stderr "    %d -> %d, %d, %d \n"
        num  a b c 
  ) neighbours;
  
  Printf.fprintf stderr "Next position: %d\n" t.proposed_robber_node;
  flush stderr;
  []
    
let _ = 
  register_robber "robber13" 
    "same as robber9 with sum_to_cops v2"
    { 
    Robber1.basic_robber with
    robber_inform = inform;
    robber_sumcops = Robber_misc.sum_to_cops2;
    robber_bribe = BribeClosest 3;
  }
  