(* $Id: robber2.ml,v 1.3.4.2 2005/07/10 12:07:17 lefessan Exp $ *)


(* 

TODO list:

* Si on arrive a une banque en n mod 8 <> 0, on peut attendre qques tours
pour rentrer dans la banque uniquement en n mod 8 = 0.
* Si un flic reste a cote d'une banque, on ne s'interesse pas a cette banque.
* Ne pas aller systematiquement vers la banque la plus proche.
* Fixer les destinations une fois pour toute
*)

open Args
open Message
open Robber_types  
open Robber_topology
open Robber
  
(* [init world_skeleton] initialises the state with the current world
skeleton *)
  

  
let initialized = ref false

open Graphics
  
  
  
(* [move world] : compute next robber move *)


let init t = ()
  
let compute_node_value t w i =
  let nod = t.nodes.(i) in
  
  
  let cops_proximity = Robber_misc.compute_cops_proximity t w 4 i in
  let is_in_culdesac = 
    if cops_proximity > 0 then nod.node_danger else 0 in 
  
  let arrested_risk = 
    let risk = ref 0 in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car ->
          if t.car_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
              Printf.fprintf stderr "Arrested risk by car cop in %d\n" i;
              incr risk
            end
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
              Printf.fprintf stderr "Arrested risk by foot cop in %d\n" i;
              incr risk
            end
      | Robber -> assert false
    done;
    !risk
  in
  
  let smelled_risk = 
    let risk = ref (2 * arrested_risk) in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car -> ()
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) = 2 then
            incr risk 
      | Robber -> assert false
    done;
    !risk    
  in
  
  let bank_distance = Robber_misc.bank_distance t w i in 
  
  let node_degree = List.length t.foot_edges.(i) in
  
  (arrested_risk, is_in_culdesac, smelled_risk, 
    cops_proximity/3, bank_distance, node_degree)
  
let inform t w = 
  
  let neighbours = t.robber_node :: t.foot_edges.(t.robber_node) in
  let neighbours = List.map (fun i ->
        compute_node_value t w i, i        
    ) neighbours in
  
  let neighbours = List.sort compare neighbours in
  t.proposed_robber_node <- snd (List.hd neighbours);
  
  Printf.fprintf stderr "\nCurrent position: %d\n" t.robber_node;
  Printf.fprintf stderr "Move sorted:\n";
  List.iter (fun ( (a,b,c,d,e,f), num) ->
      Printf.fprintf stderr "    %d -> %d, %d, %d, %d, %d, %d \n"
        num  a b c d e f
  ) neighbours;
  Printf.fprintf stderr "Next position: %d\n" t.proposed_robber_node;
  flush stderr;
  []
  
let _ = 
  register_robber "robber2" 
    "first robber"
    {
    Robber1.basic_robber with
    robber_inform = inform;
    robber_sumcops = Robber_misc.sum_to_cops1;
  }
