(* $Id: robber6.ml,v 1.6.4.2 2005/07/10 12:07:17 lefessan Exp $ *)


(* 

TODO list:

* Si on arrive a une banque en n mod 8 <> 0, on peut attendre qques tours
pour rentrer dans la banque uniquement en n mod 8 = 0.
* Si un flic reste a cote d'une banque, on ne s'interesse pas a cette banque.
* Ne pas aller systematiquement vers la banque la plus proche.
* Fixer les destinations une fois pour toute
*)

open Args
open Message
open Robber_types
open Robber_topology
open Robber

let objectives = ref [198;126;198;126;198;126;198;126;198;126]
  
let compute_cops_proximity t w dist_threshold i = 
  let sum = ref 0 in
  for cop = 0 to t.ncops - 1 do
    match w.cops_ptypes.(cop) with
      Cop_car ->
        if t.car_dist.(w.cops_nodes.(cop)).(i) < dist_threshold then begin
(*            Printf.fprintf stderr "Car cop closed to %d : %d\n" 
              i t.car_dist.(w.cops_nodes.(cop)).(i); *)
            incr sum
          end
    | Cop_foot ->
        if t.foot_dist.(w.cops_nodes.(cop)).(i) < dist_threshold then begin
(*            Printf.fprintf stderr "Foot cop closed to %d : %d\n" 
              i t.foot_dist.(w.cops_nodes.(cop)).(i); *)
            incr sum
          end
    | Robber -> assert false
  done;
  !sum
  
  
let compute_node_value t w valuate_path i =
  let nod = t.nodes.(i) in
    
  let arrested_risk = 
    let risk = ref 0 in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car ->
          if t.car_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
(*              Printf.fprintf stderr "Arrested risk by car cop in %d\n" i; *)
              incr risk
            end
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) <= 1 then begin
(*              Printf.fprintf stderr "Arrested risk by foot cop in %d\n" i; *)
              incr risk
            end
      | Robber -> assert false
    done;
    !risk
  in
  
  let smelled_risk = 
    let risk = ref (2 * arrested_risk) in
    for cop = 0 to t.ncops - 1 do
      match w.cops_ptypes.(cop) with
        Cop_car -> ()
      | Cop_foot ->
          if t.foot_dist.(w.cops_nodes.(cop)).(i) = 2 then
            incr risk 
      | Robber -> assert false
    done;
    !risk    
  in

  let paths = valuate_path i in
  
  (arrested_risk, 
    smelled_risk, 
    t.foot_dist.(i).(List.hd !objectives), 
    -paths )
  
let inform t w = 

  if w.state_num = 170 then
    objectives := 198 :: !objectives;
  
  begin
    match !objectives with
      [_] | [] -> ()
    | x :: tail ->
        if x = t.robber_node then 
          objectives := tail
  end;
  
  let valuate_path = 
    Robber_misc.sum_to_cops_from t w [t.robber_node] in

  let valuate_path = 
    Robber_misc.path_from t w 
      (fun len i -> 
        let dist = t.nodes.(i).node_copdist in
        20 * (max (dist -  len) 0)
        
        ) (fun len i -> t.nodes.(i).node_copsum) 6 [t.robber_node]  in
  
  let neighbours = t.robber_node :: t.foot_edges.(t.robber_node) in 
  let neighbours = List.map (fun i ->
        compute_node_value t w valuate_path i, i        
    ) neighbours in
  
  let neighbours = List.sort compare neighbours in
  let next_node = snd (List.hd neighbours) in
  t.proposed_robber_node <- next_node;
  
  Printf.fprintf stderr "\nCurrent position: %d\n" t.robber_node;
  Printf.fprintf stderr "Move sorted:\n";
  List.iter (fun ( (a,b,c,d), num) ->
      Printf.fprintf stderr "    %d -> %d, %d, %d, %d \n"
        num  a b c d 
  ) neighbours;
  Printf.fprintf stderr "Next position: %d\n" next_node;
  flush stderr;
  []
  
let _ = 
  register_robber "robber6" 
  "hide in corner"
    {
    Robber1.basic_robber with
    robber_inform = inform;
    robber_sumcops = Robber_misc.sum_to_cops2;    
  }
