(* $Id: robber_bribing.ml,v 1.1.2.5 2005/07/10 12:07:17 lefessan Exp $ *)

open Args
open Message
open Robber_types  
open Robber_topology
open Robber
  
let has_bribed = ref false
  
let waiting t s =
  Printf.fprintf stderr "Robber_bribing.waiting...\n"; 
  if t.stolen >= 50 then
    let should_bribe =
      match s.state_robber.robber_bribe with
        BribeOften -> s.state_known
      | BribeClosest bribe_dist ->
          let should_bribe = ref false in
          for c = 0 to t.ncops - 1 do
            if not s.cops_bribed.(c) then
              let dists = match s.cops_ptypes.(c) with
                  Cop_car -> t.car_dist
                | Cop_foot -> t.foot_dist
                | _ -> assert false
              in
              let pos = s.cops_nodes.(c) in
(* If a non-bribed cop is too close from our next position, 
  try to corrupt it ! *)
              if dists.(pos).(t.proposed_robber_node) < bribe_dist then
                should_bribe := true
          done;
          !should_bribe
    in
    if should_bribe then  Bribe else Nobribe
  else
    Nobribe
    
    
(* Try to push the closest cop *)
let choose_push t s =
  let closest = ref 0 in
  let closest_dist = ref 1000 in
  for c = 0 to t.ncops - 1 do
    let dist =
      match s.cops_ptypes.(c) with
        Cop_car -> 
          t.car_dist.(s.cops_nodes.(c)).(t.proposed_robber_node)
      | Cop_foot -> 
          t.foot_dist.(s.cops_nodes.(c)).(t.proposed_robber_node)
      | _ -> assert false
    in
    if dist < !closest_dist then begin
        closest_dist := dist;
        closest := c
      end
  done;
  let c = !closest in
  let p = s.cops_nodes.(!closest) in
  let (edges, dists) = match s.cops_ptypes.(c) with
    | Cop_foot -> t.foot_edges.(s.cops_nodes.(c)), t.foot_dist
    | Cop_car -> t.car_edges.(s.cops_nodes.(c)), t.car_dist
    | _ -> assert false
  in
  let list = List.map (fun e ->
        - dists.(e).(t.proposed_robber_node), e) edges in
  let list = List.sort compare list in
  List.iter (fun (dist, e) ->
      Printf.fprintf stderr "   To %d, new dist: %d\n"
        e (-dist);
  ) list;
  let first = snd (List.hd list) in
  Printf.fprintf stderr "PUSH %s IN %d\n" t.cops_names.(!closest)
  first;
  t.stolen <- t.stolen - 50;  
  Push {
    push_bot = t.cops_names.(!closest);
    push_loc = t.nodes.(first).node_loc;
  }
  
let moving t s offered =
  Printf.fprintf stderr "Robber_bribing.moving...\n"; 
  let bribe =
    match offered with
      NotTelling -> 
        Printf.fprintf stderr "DID NOT BRIBE\n"; 
        Nothing
    | Offered [] ->
        Printf.fprintf stderr "DID BRIBE: NO OFFER...\n"; 
        if not !has_bribed then 
          choose_push t s
        else
          Choose []
    | Offered list ->
        List.iter (fun name ->
            Printf.fprintf stderr "DO BRIBE: %s proposed...\n" name) list; 

(*
1) Try to corrupt all close cops.
2) If one of them can not be corrupted, corrupt all the cops even far ones.
*)
        let list =
          match s.state_robber.robber_bribe with 
            BribeOften -> list
          | BribeClosest bribe_dist ->
              let can_corrupt = ref [] in
              let cannot_corrupt_all = ref false in
              for c = 0 to t.ncops - 1 do
                if not s.cops_bribed.(c) then 
                  let dist =
                    match s.cops_ptypes.(c) with
                      Cop_car -> 
                        t.car_dist.(s.cops_nodes.(c)).(t.proposed_robber_node)
                    | Cop_foot -> 
                        t.foot_dist.(s.cops_nodes.(c)).(t.proposed_robber_node)
                    | _ -> assert false
                  in
                  if dist < bribe_dist then 
                    let name = t.cops_names.(c) in
                    if List.mem name list then
                      can_corrupt := name :: !can_corrupt
                    else
                      cannot_corrupt_all := true
              done;
              if !cannot_corrupt_all then list else
                  !can_corrupt
        in              
        if list <> [] then has_bribed := true;
        Choose list
  in
  t.proposed_robber_bribe <- bribe
  