(* $Id: robber_draw.ml,v 1.9.2.3 2005/07/10 12:07:17 lefessan Exp $ *)

open Args
open Message
open Graphics
open Robber_types
open Robber_topology
  
let width = ref 0 
let height = ref 0
let max_x = ref 0 
let max_y = ref 0 

let interactive = ref true

let foot_color = blue
let car_color = magenta
  
let tox x = (10+x) * !width / !max_x
let toy y = !height - (10+y) * !height / !max_y

let x11_possibles = ref true


let score_bank t w node =
  match node.node_tag with
  | Ordinary -> 0
  | Robber_start -> 0
  | Hq -> 0
  | Bank bank -> 
      let i = bank.bank_num in
      let all_cops = w.bank_all_cops.(i) in
      let foot_cops = w.bank_foot_cops.(i) in
      let car_cops = w.bank_car_cops.(i) in
      let good_bank = 
        if all_cops.(5) < 1 then begin
            2
          end else
        if foot_cops.(7) < 1 && car_cops.(2) < 1 then begin
            2
          end else
        if all_cops.(5) < 2 then begin
            1
          end
        else 
        if all_cops.(5) < 3 then
          0
        else begin
(*    if cops_proximity5 = 5 then begin
        Printf.fprintf stderr "----------   NO HOPE ------------\n";
        1
end else  *)
            -1
          end
      in
      10 * good_bank
      
      
  
let draw_mapx t w =
  if !x11_map then begin
      
      clear_graph ();
      
      Array.iter (fun node ->
          
          let x = node.node_x in
          let y = node.node_y in
          
          let xx = tox x in
          let yy = toy y in
          
          begin
            match node.node_tag with
              Ordinary -> 
                set_color black;
                draw_circle xx yy 2;
                if node.node_danger > 0 then begin
                    set_color red;
                    fill_poly [| xx-3, yy-3; xx-3, yy+3; xx+3, yy+3; xx+3, yy-3|];
                  end
            
            | Robber_start -> 
                set_color black;
                draw_circle xx yy 5;
                set_color red;
                fill_circle xx yy 5;
            | Bank _ -> 
                set_color yellow;
                fill_circle xx yy 5;
                let n = score_bank t w node in 
                set_color black;
                draw_circle xx yy 5;
                moveto (xx+10) (yy-10);
                draw_string (string_of_int n)
                
                
            | Hq -> 
                set_color black;
                draw_circle xx yy 5;
                set_color blue;
                fill_circle xx yy 5;
          end;
          
          
          List.iter (fun dst ->
              
              let dst = t.nodes.(dst) in
              let x' = dst.node_x in
              let y' = dst.node_y in
              
              let xx' = tox x' in
              let yy' = toy y' in
              
              set_color green;
              moveto (xx-1) (yy-1);
              lineto (xx'-1) (yy'-1)
          
          ) t.foot_edges.(node.node_num);

          if !x11_num then begin
              set_color black;
              moveto (xx+5) (yy+5);
              draw_string (string_of_int node.node_num);
            end;
          if !x11_car then begin
              set_color car_color;
              moveto (xx+5) (yy-15);
              draw_string (string_of_int 
                t.car_dist.(node.node_num).(
                  if !x11_dist >= 0 then !x11_dist else t.robber_node));
            end;
          if !x11_foot then begin
              set_color foot_color;
              moveto (xx-15) (yy-15);
              draw_string (string_of_int 
                  t.foot_dist.(node.node_num).(
                  if !x11_dist >= 0 then !x11_dist else t.robber_node));
            end;

          if !x11_road then
          List.iter (fun dsti ->
              set_color black;
              
              let dst = t.nodes.(dsti) in
              let x' = dst.node_x in
              let y' = dst.node_y in
              
              let xx' = tox x' in
              let yy' = toy y' in
              if not (List.mem dsti t.foot_edges.(node.node_num)) then begin
                  
                  let x1 = xx + 1 in
                  let y1 = yy + 1 in
                  let x2 = xx' + 1 in
                  let y2 = yy' + 1 in
                  moveto x1 y1;
(*               lineto x2 y2; *)
                  
                  let width =
                    if List.mem node.node_num t.car_edges.(dsti) then
                      if dsti < node.node_num then 2 else 0
                    else 1
                  in
                  if width > 0 then              
                    let diffy = (y1-y2)/13 in
                    let diffx = (x2-x1)/13 in

(*
                let diffy = if diffy * diffy > 100 then 
                    diffy / 2 
                  else diffy in
                let diffx = if diffx * diffx > 100 then 
                    diffx / 2
                  else diffx in *)
                    let diffx, diffy = 
                      if (node.node_num - dsti) mod 2 = 0 then diffx, diffy else
                        -diffx, -diffy in
                    set_line_width width;
                    curveto (x1-diffy ,y1- diffx ) (x2-diffy,y2-diffx) (x2,y2);
                    set_line_width 1;
                end;            
              let xxx' = xx + (xx' - xx) * 9 / 10 in
              let yyy' = yy + (yy' - yy) * 9 / 10 in
              fill_circle (xxx'+1) (yyy'+1) 2;
          
          ) t.car_edges.(node.node_num);
      ) t.nodes;  
    end

    
let draw_statex t s = 
  if !x11_map then begin
      draw_mapx t s;
      moveto 2 2;
      draw_string (Printf.sprintf "Round: %3d  Robberies: %2d Stolen: %4d"
          s.state_num t.robberies t.stolen);
      List2.iteri (fun i s ->
          moveto 300 (2 + 20 * i);
          draw_string s
      ) s.state_messages;
      
      Array.iteri (fun i n ->
          let copnod = t.nodes.(n) in
          set_color (match s.cops_ptypes.(i) with
              Cop_foot -> foot_color
            | Cop_car -> car_color 
            | _ -> assert false
          );
          draw_circle (tox copnod.node_x) (toy copnod.node_y) 6;
          fill_circle (tox copnod.node_x) (toy copnod.node_y) 6;
      ) s.cops_nodes;
      let robnod = t.nodes.(t.robber_node) in
      set_color red;
      fill_circle (tox robnod.node_x) (toy robnod.node_y) 6;
      draw_circle (tox robnod.node_x) (toy robnod.node_y) 6;
      
      if !x11_copsum then       
        Array.iter (fun node ->
            
            let x = node.node_x in
            let y = node.node_y in
            
            let xx = tox x in
            let yy = toy y in
            set_color blue;
            moveto (xx-15) (yy-15);
            draw_string (string_of_int node.node_copsum)
        ) t.nodes;
      
      if !x11_copdist then       
        Array.iter (fun node ->
            
            let x = node.node_x in
            let y = node.node_y in
            
            let xx = tox x in
            let yy = toy y in
            set_color blue;
            moveto (xx-15) (yy-15);
            draw_string (string_of_int node.node_copdist)
        ) t.nodes;
      
      if !x11_possibles && Array.length s.state_possibles <> 0 then begin
          set_line_width 3;
          set_color red;
          Array.iter (fun node ->
              
              let x = node.node_x in
              let y = node.node_y in
              
              let xx = tox x in
              let yy = toy y in
              if s.state_possibles.(node.node_num) then
                draw_circle xx yy 7
          ) t.nodes;
          set_line_width 1;          
        end;
      
      if !x11_dens then begin
            Array.iter (fun node ->
                
                let x = node.node_x in
                let y = node.node_y in
                
                let xx = tox x in
                let yy = toy y in
                set_color magenta;
                moveto (xx-15) (yy-15);
                draw_string (string_of_int node.node_density)
            ) t.nodes
          end;
        
    end
    
let init t = 
  if !x11_map then
    let _x11 = open_graph ":0 1000x700" in
    width := size_x ();
    height := size_y ();
    
    Array.iter (fun node ->
        
        let x = node.node_x in
        let y = node.node_y in
        
        max_x := max !max_x x;
        max_y := max !max_y y;
    ) t.nodes;
    
    max_x := (10 + !max_x) * 11 / 10;
    max_y := (10 + !max_y) * 11 / 10
    
let wait_delay = ref 0.5
let count = ref 0
  
  
let close () = 
  if !x11_map then
    close_graph ()
  
let wait t s =     
  (if !count > 0 then decr count);
  if !x11_map then
    if !interactive && !count = 0 then begin
        let continue = ref true in
        while !continue do
          draw_statex t s;
          match read_key () with
            'q' -> 
              close ();
              x11_map := false;
              continue := false;
              wait_delay := 0.;
              interactive := false
          | 'c' -> 
              continue := false;
              wait_delay := 0.;
              interactive := false
          | 'n' -> 
              count := 10; 
              continue := false
          | 'N' ->
              x11_num := not !x11_num
          | 'F' ->
              x11_foot := not !x11_foot
          | 'C' ->
              x11_car := not !x11_car
          | 'R' ->
              x11_road := not !x11_road
          | 'S' ->
              x11_copsum := not !x11_copsum
          | 'D' ->
              x11_copdist := not !x11_copdist
          | 'd' ->
              x11_dens := not !x11_dens
          | 'p' ->
              x11_possibles := not !x11_possibles
          | _ -> 
              continue := false
        done
      end else begin
        draw_statex t s;        
        let rec iter delay =
          if delay > 0. then
            let begin_time = Unix.gettimeofday () in
            try
              ignore (Unix.select [] [] [] delay)
            with Unix.Unix_error(Unix.EINTR,_,_) ->
                let end_time = Unix.gettimeofday () in
                iter (delay -. (end_time -. begin_time))
        in
        iter !wait_delay
      end
      
