(* $Id: robber_possibles.ml,v 1.2.2.4 2005/07/10 12:52:31 lefessan Exp $ *)

open Args
open Message
open Robber_types
open Robber_topology

let max_nnodes = 500  

type schedule = state array

let possibles = ref None
let evidences = Array.create max_nnodes []

let cardinal t =
  let n = ref 0 in
  for i = 0 to Array.length t - 1 do
    if t.(i) then incr n
  done;
  !n

let propagate t previous_table s new_table =
  if s.state_known then ()

let steps t possibles from_num to_num = 
  Printf.fprintf stderr "STEPS %d -> %d\n" from_num to_num; 
  for num = from_num to to_num do
    if num mod 2 = 0 then
      let s = possibles.(num) in
      let table = Array.create !nnodes false in
      s.state_possibles <- table;
      if s.state_known then begin
          table.(s.state_robbernode) <- true;
        end else begin
          let previous_table = possibles.(num - 2).state_possibles in
(* Expand previous possibles *)
          for i = 0 to !nnodes - 1 do
            if previous_table.(i) then begin
                List.iter (fun e ->
                    table.(e) <- true
                ) t.foot_edges.(i);
                table.(i) <- true
              end
          done;
(* Restrict by smell *)
          let robber_node = s.state_robbernode in
          for c = 0 to t.ncops - 1 do
            let cop_node = s.cops_nodes.(c) in
            let cop_ptype = s.cops_ptypes.(c) in
            let (dists, threshold) =
              match cop_ptype with
                Cop_foot -> 
                  let dists = t.foot_dist in
                  let threshold = 2 in
                  dists, threshold
              
              | Cop_car -> 
                  let dists = t.car_dist in
                  let threshold = 1 in
                  dists, threshold
              
              | _ -> assert false
            in
            
            let dist = dists.(cop_node).(robber_node) in
            if dist <= threshold then
              for i = 0 to !nnodes - 1 do
                if dists.(cop_node).(i) <> dist then
                  table.(i) <- false
              done
            else
            for i = 0 to !nnodes - 1 do
              if dists.(cop_node).(i) <= threshold then
                table.(i) <- false
            done
          done;
          
          Printf.fprintf stderr "POSSIBLES AT %d CHANGED FROM %d to %d\n"
            num (cardinal previous_table) (cardinal table)
        end
  done;
  Printf.fprintf stderr "STEPS DONE\n"; 
  ()

let get_possibles s =
  match !possibles with
      Some p -> p
    | None ->
        let dummy = {s with state_known = false } in
        let p = Array.create 200 dummy in
        possibles := Some p;
        p
  
let inform t s =
  Printf.fprintf stderr "Robber_possibles.inform...\n"; 
  let possibles = get_possibles s in  
  if s.state_num = 0 then s.state_known <- true;
  possibles.(s.state_num) <- s;
  s.state_robbernode <- t.robber_node;
  
  if s.state_num mod 8 = 0 then begin
      evidences.(t.robber_node) <- s.state_num :: evidences.(t.robber_node);
    end;
  
  Array.iter (fun b ->
      if t.robber_node = b.bank_node then
        s.state_known <- true
  ) t.banks;
  
  let previous_step = ref s.state_num in
  Array.iter (fun pos ->
      List.iter (fun num ->
          let how_long = s.state_num - num in
          if how_long < 24 then begin
              possibles.(num).state_known <- true;
              Printf.fprintf stderr 
                "EVIDENCE FOUND AT %d FOR WORLD %d\n" pos num;
              s.state_messages <- 
                (Printf.sprintf "EVIDENCE FOUND AT %d FOR WORLD %d"
                  pos num
              ) :: s.state_messages;
              
              
              let new_evidence = ref true in
              for i = num + 1 to s.state_num do
                if possibles.(i).state_known then begin
                    Printf.fprintf stderr "POS %d ALREADY KNOWN\n" i;
                    new_evidence := false
                  end
              done;
              Printf.fprintf stderr "NEW EVIDENCE %b %d %d\n"
                !new_evidence !previous_step num;
              if !new_evidence && !previous_step > num then
                previous_step := num
            end
      ) evidences.(pos)
  ) s.cops_nodes;
  Printf.fprintf stderr "previous_step = %d\n" !previous_step; 
  steps t possibles !previous_step s.state_num
  
